program RWEmig_
  use sep
  use wemig_

  implicit none
  integer :: method
  integer :: nx,nz,nw, iz
  real    :: dx,dz,dw
  real    :: oz,ox,ow
  character(len=128)   :: name
  real,    allocatable :: aa(:,:),bb(:,:)
  real,    allocatable :: a0(:),  b0(:)
  complex, pointer     :: dat(:,:) !! data
  real,    pointer     :: img(:,:) !! image

  real,    allocatable :: slo(:,:),sl0(:) !! true slowness
  real,    allocatable :: alp(:,:),al0(:) !! coordinate system slowness
  real,    allocatable :: geo(:,:),ge0(:) !! geometrical spreading

  call sep_init()
  call from_param("method",method,0)

  call from_history("n1",nx,1)
  call from_history("o1",ox)
  call from_history("d1",dx)
  call from_history("n2",nw,1)
  call from_history("o2",ow)
  call from_history("d2",dw)

  name="ajs"
  call from_aux(name,"n1",nz)
  call from_aux(name,"o1",oz)
  call from_aux(name,"d1",dz)

  write(0,*) nx,ox,dx
  write(0,*) nz,oz,dz
  write(0,*) nw,ow,dw

  call to_history("n1",nz)
  call to_history("o1",oz)
  call to_history("d1",dz)
  call to_history("n2",nx)
  call to_history("o2",ox)
  call to_history("d2",dx)

  call to_history("esize",4)

  allocate(aa(nz,nx),a0(nz))
  allocate(bb(nz,nx),b0(nz))
  allocate(dat(nx,nw))
  allocate(img(nz,nx))

  allocate(slo(nz,nx),sl0(nz))
  allocate(alp(nz,nx),al0(nz))
  allocate(geo(nz,nx),ge0(nz))

  call sreed("ajs",alp,size(alp)*4)
  call sreed("ajs",geo,size(geo)*4)
  call sreed("ajs",slo,size(slo)*4)
  write(0,*) "ALP",minval(alp),maxval(alp)
  write(0,*) "GEO",minval(geo),maxval(geo)
  write(0,*) "SLO",minval(slo),maxval(slo)

  aa=slo*alp
  bb=alp/geo
  !----------------------------------------------------------------
  do iz=1,nz
     a0(iz)= maxval(aa(iz,:))
     b0(iz)= minval(bb(iz,:))
  end do
  write(0,*) "aa",minval(aa),maxval(aa)
  write(0,*) "bb",minval(bb),maxval(bb)

  call wemig_init(nz,nx,nw,dz,dx,dw,oz,ox,ow,method)
  call sep_read(dat)
  img=0.0
  call wemig(dat,img,aa,bb,a0,b0)
  call sep_write(img)
  !----------------------------------------------------------------  

  call sep_close()
  deallocate(dat,img,alp,al0,geo,ge0,slo,sl0)
end program RWEmig_
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
